@@ -45,6 +45,7 @@
 #include "utils/text.h"
 #include "utils/Table.h"
 #include "utils/GetOpts.h"
+#include "utils/HumanString.h"
 
 #include "cleanup.h"
 #include "errors.h"
@@ -166,8 +167,8 @@ MyFiles::bulk_process(FILE* file, std::function<void(File& file)> callback)
 void
 help_list_configs()
 {
-    cout << _("  List configs:") << endl
-	 << _("\tsnapper list-configs") << endl
+    cout << _("  List configs:") << '\n'
+	 << _("\tsnapper list-configs") << '\n'
 	 << endl;
 }
 
@@ -205,12 +206,12 @@ command_list_configs(ProxySnappers* snappers, ProxySnapper*)
 void
 help_create_config()
 {
-    cout << _("  Create config:") << endl
-	 << _("\tsnapper create-config <subvolume>") << endl
-	 << endl
-	 << _("    Options for 'create-config' command:") << endl
-	 << _("\t--fstype, -f <fstype>\t\tManually set filesystem type.") << endl
-	 << _("\t--template, -t <name>\t\tName of config template to use.") << endl
+    cout << _("  Create config:") << '\n'
+	 << _("\tsnapper create-config <subvolume>") << '\n'
+	 << '\n'
+	 << _("    Options for 'create-config' command:") << '\n'
+	 << _("\t--fstype, -f <fstype>\t\tManually set filesystem type.") << '\n'
+	 << _("\t--template, -t <name>\t\tName of config template to use.") << '\n'
 	 << endl;
 }
 
@@ -262,8 +263,8 @@ command_create_config(ProxySnappers* snappers, ProxySnapper*)
 void
 help_delete_config()
 {
-    cout << _("  Delete config:") << endl
-	 << _("\tsnapper delete-config") << endl
+    cout << _("  Delete config:") << '\n'
+	 << _("\tsnapper delete-config") << '\n'
 	 << endl;
 }
 
@@ -285,8 +286,8 @@ command_delete_config(ProxySnappers* snappers, ProxySnapper*)
 void
 help_get_config()
 {
-    cout << _("  Get config:") << endl
-	 << _("\tsnapper get-config") << endl
+    cout << _("  Get config:") << '\n'
+	 << _("\tsnapper get-config") << '\n'
 	 << endl;
 }
 
@@ -324,8 +325,8 @@ command_get_config(ProxySnappers* snappers, ProxySnapper* snapper)
 void
 help_set_config()
 {
-    cout << _("  Set config:") << endl
-	 << _("\tsnapper set-config <configdata>") << endl
+    cout << _("  Set config:") << '\n'
+	 << _("\tsnapper set-config <configdata>") << '\n'
 	 << endl;
 }
 
@@ -349,12 +350,13 @@ command_set_config(ProxySnappers* snappers, ProxySnapper* snapper)
 void
 help_list()
 {
-    cout << _("  List snapshots:") << endl
-	 << _("\tsnapper list") << endl
-	 << endl
-	 << _("    Options for 'list' command:") << endl
-	 << _("\t--type, -t <type>\t\tType of snapshots to list.") << endl
-	 << _("\t--all-configs, -a\t\tList snapshots from all accessible configs.") << endl
+    cout << _("  List snapshots:") << '\n'
+	 << _("\tsnapper list") << '\n'
+	 << '\n'
+	 << _("    Options for 'list' command:") << '\n'
+	 << _("\t--type, -t <type>\t\tType of snapshots to list.") << '\n'
+	 << _("\t--disable-used-space\t\tDisable showing used space.") << '\n'
+	 << _("\t--all-configs, -a\t\tList snapshots from all accessible configs.") << '\n'
 	 << endl;
 }
 
@@ -362,7 +364,7 @@ enum ListMode { LM_ALL, LM_SINGLE, LM_PRE_POST };
 
 
 void
-list_from_one_config(ProxySnapper* snapper, ListMode list_mode);
+list_from_one_config(ProxySnapper* snapper, ListMode list_mode, bool show_used_space);
 
 
 void
@@ -370,6 +372,7 @@ command_list(ProxySnappers* snappers, ProxySnapper*)
 {
     const struct option options[] = {
 	{ "type",		required_argument,	0,	't' },
+	{ "disable-used-space", no_argument,            0,      0 },
 	{ "all-configs",	no_argument,		0,	'a' },
 	{ 0, 0, 0, 0 }
     };
@@ -382,6 +385,7 @@ command_list(ProxySnappers* snappers, ProxySnapper*)
     }
 
     ListMode list_mode = LM_ALL;
+    bool show_used_space = true;
 
     GetOpts::parsed_opts::const_iterator opt;
 
@@ -400,6 +404,11 @@ command_list(ProxySnappers* snappers, ProxySnapper*)
 	}
     }
 
+    if ((opt = opts.find("disable-used-space")) != opts.end())
+    {
+       show_used_space = false;
+    }
+
     vector<string> tmp;
 
     if ((opt = opts.find("all-configs")) == opts.end())
@@ -426,14 +435,51 @@ command_list(ProxySnappers* snappers, ProxySnapper*)
                  << snapper->getConfig().getSubvolume() << endl;
         }
 
-        list_from_one_config(snapper, list_mode);
+        list_from_one_config(snapper, list_mode, show_used_space);
     }
 }
 
 
 void
-list_from_one_config(ProxySnapper* snapper, ListMode list_mode)
+list_from_one_config(ProxySnapper* snapper, ListMode list_mode, bool show_used_space)
 {
+    const ProxySnapshots& snapshots = snapper->getSnapshots();
+
+    ProxySnapshots::const_iterator default_snapshot = snapshots.getDefault();
+    ProxySnapshots::const_iterator active_snapshot = snapshots.getActive();
+
+    if (list_mode != LM_ALL && list_mode != LM_SINGLE)
+	show_used_space = false;
+
+    if (show_used_space)
+    {
+	try
+	{
+	    snapper->calculateUsedSpace();
+	}
+	catch (const QuotaException& e)
+	{
+	    SN_CAUGHT(e);
+
+	    show_used_space = false;
+	}
+    }
+
+    auto format_num = [&snapshots, default_snapshot, active_snapshot](const ProxySnapshot& snapshot) -> string {
+	bool is_default = default_snapshot != snapshots.end() && default_snapshot->getNum() == snapshot.getNum();
+	bool is_active = active_snapshot != snapshots.end() && active_snapshot->getNum() == snapshot.getNum();
+	static const char sign[2][2] = { { ' ', '-' }, { '+', '*' } };
+	return decString(snapshot.getNum()) + sign[is_default][is_active];
+    };
+
+    auto format_date = [](const ProxySnapshot& snapshot) -> string {
+	return snapshot.isCurrent() ? "" : datetime(snapshot.getDate(), utc, iso);
+    };
+
+    auto format_used_space = [](const ProxySnapshot& snapshot) -> string {
+	return snapshot.isCurrent() ? "" : byte_to_humanstring(snapshot.getUsedSpace(), 2);
+    };
+
     Table table;
 
     switch (list_mode)
@@ -441,25 +487,28 @@ list_from_one_config(ProxySnapper* snapper, ListMode list_mode)
 	case LM_ALL:
 	{
 	    TableHeader header;
+	    header.add(_("#"), TableAlign::RIGHT);
 	    header.add(_("Type"));
-	    header.add(_("#"));
-	    header.add(_("Pre #"));
+	    header.add(_("Pre #"), TableAlign::RIGHT);
 	    header.add(_("Date"));
 	    header.add(_("User"));
+	    if (show_used_space)
+		header.add(_("Used Space"), TableAlign::RIGHT);
 	    header.add(_("Cleanup"));
 	    header.add(_("Description"));
 	    header.add(_("Userdata"));
 	    table.setHeader(header);
 
-	    const ProxySnapshots& snapshots = snapper->getSnapshots();
 	    for (const ProxySnapshot& snapshot : snapshots)
 	    {
 		TableRow row;
+		row.add(format_num(snapshot));
 		row.add(toString(snapshot.getType()));
-		row.add(decString(snapshot.getNum()));
 		row.add(snapshot.getType() == POST ? decString(snapshot.getPreNum()) : "");
-		row.add(snapshot.isCurrent() ? "" : datetime(snapshot.getDate(), utc, iso));
+		row.add(format_date(snapshot));
 		row.add(username(snapshot.getUid()));
+		if (show_used_space)
+		    row.add(format_used_space(snapshot));
 		row.add(snapshot.getCleanup());
 		row.add(snapshot.getDescription());
 		row.add(show_userdata(snapshot.getUserdata()));
@@ -471,23 +520,26 @@ list_from_one_config(ProxySnapper* snapper, ListMode list_mode)
 	case LM_SINGLE:
 	{
 	    TableHeader header;
-	    header.add(_("#"));
+	    header.add(_("#"), TableAlign::RIGHT);
 	    header.add(_("Date"));
 	    header.add(_("User"));
+	    if (show_used_space)
+		header.add(_("Used Space"), TableAlign::RIGHT);
 	    header.add(_("Description"));
 	    header.add(_("Userdata"));
 	    table.setHeader(header);
 
-	    const ProxySnapshots& snapshots = snapper->getSnapshots();
 	    for (const ProxySnapshot& snapshot : snapshots)
 	    {
 		if (snapshot.getType() != SINGLE)
 		    continue;
 
 		TableRow row;
-		row.add(decString(snapshot.getNum()));
-		row.add(snapshot.isCurrent() ? "" : datetime(snapshot.getDate(), utc, iso));
+		row.add(format_num(snapshot));
+		row.add(format_date(snapshot));
 		row.add(username(snapshot.getUid()));
+		if (show_used_space)
+		    row.add(format_used_space(snapshot));
 		row.add(snapshot.getDescription());
 		row.add(show_userdata(snapshot.getUserdata()));
 		table.add(row);
@@ -498,15 +550,14 @@ list_from_one_config(ProxySnapper* snapper, ListMode list_mode)
 	case LM_PRE_POST:
 	{
 	    TableHeader header;
-	    header.add(_("Pre #"));
-	    header.add(_("Post #"));
+	    header.add(_("Pre #"), TableAlign::RIGHT);
+	    header.add(_("Post #"), TableAlign::RIGHT);
 	    header.add(_("Pre Date"));
 	    header.add(_("Post Date"));
 	    header.add(_("Description"));
 	    header.add(_("Userdata"));
 	    table.setHeader(header);
 
-	    const ProxySnapshots& snapshots = snapper->getSnapshots();
 	    for (ProxySnapshots::const_iterator it1 = snapshots.begin(); it1 != snapshots.end(); ++it1)
 	    {
 		if (it1->getType() != PRE)
@@ -520,8 +571,8 @@ list_from_one_config(ProxySnapper* snapper, ListMode list_mode)
 		const ProxySnapshot& post = *it2;
 
 		TableRow row;
-		row.add(decString(pre.getNum()));
-		row.add(decString(post.getNum()));
+		row.add(format_num(pre));
+		row.add(format_num(post));
 		row.add(datetime(pre.getDate(), utc, iso));
 		row.add(datetime(post.getDate(), utc, iso));
 		row.add(pre.getDescription());
@@ -539,17 +590,17 @@ list_from_one_config(ProxySnapper* snapper, ListMode list_mode)
 void
 help_create()
 {
-    cout << _("  Create snapshot:") << endl
-	 << _("\tsnapper create") << endl
-	 << endl
-	 << _("    Options for 'create' command:") << endl
-	 << _("\t--type, -t <type>\t\tType for snapshot.") << endl
-	 << _("\t--pre-number <number>\t\tNumber of corresponding pre snapshot.") << endl
-	 << _("\t--print-number, -p\t\tPrint number of created snapshot.") << endl
-	 << _("\t--description, -d <description>\tDescription for snapshot.") << endl
-	 << _("\t--cleanup-algorithm, -c <algo>\tCleanup algorithm for snapshot.") << endl
-	 << _("\t--userdata, -u <userdata>\tUserdata for snapshot.") << endl
-	 << _("\t--command <command>\tRun command and create pre and post snapshots.") << endl
+    cout << _("  Create snapshot:") << '\n'
+	 << _("\tsnapper create") << '\n'
+	 << '\n'
+	 << _("    Options for 'create' command:") << '\n'
+	 << _("\t--type, -t <type>\t\tType for snapshot.") << '\n'
+	 << _("\t--pre-number <number>\t\tNumber of corresponding pre snapshot.") << '\n'
+	 << _("\t--print-number, -p\t\tPrint number of created snapshot.") << '\n'
+	 << _("\t--description, -d <description>\tDescription for snapshot.") << '\n'
+	 << _("\t--cleanup-algorithm, -c <algo>\tCleanup algorithm for snapshot.") << '\n'
+	 << _("\t--userdata, -u <userdata>\tUserdata for snapshot.") << '\n'
+	 << _("\t--command <command>\t\tRun command and create pre and post snapshots.") << endl
 	 << endl;
 }
 
@@ -672,13 +723,13 @@ command_create(ProxySnappers* snappers, ProxySnapper* snapper)
 void
 help_modify()
 {
-    cout << _("  Modify snapshot:") << endl
-	 << _("\tsnapper modify <number>") << endl
-	 << endl
-	 << _("    Options for 'modify' command:") << endl
-	 << _("\t--description, -d <description>\tDescription for snapshot.") << endl
-	 << _("\t--cleanup-algorithm, -c <algo>\tCleanup algorithm for snapshot.") << endl
-	 << _("\t--userdata, -u <userdata>\tUserdata for snapshot.") << endl
+    cout << _("  Modify snapshot:") << '\n'
+	 << _("\tsnapper modify <number>") << '\n'
+	 << '\n'
+	 << _("    Options for 'modify' command:") << '\n'
+	 << _("\t--description, -d <description>\tDescription for snapshot.") << '\n'
+	 << _("\t--cleanup-algorithm, -c <algo>\tCleanup algorithm for snapshot.") << '\n'
+	 << _("\t--userdata, -u <userdata>\tUserdata for snapshot.") << '\n'
 	 << endl;
 }
 
@@ -727,15 +778,44 @@ command_modify(ProxySnappers* snappers, ProxySnapper* snapper)
 void
 help_delete()
 {
-    cout << _("  Delete snapshot:") << endl
-	 << _("\tsnapper delete <number>") << endl
-	 << endl
-	 << _("    Options for 'delete' command:") << endl
-	 << _("\t--sync, -s\t\t\tSync after deletion.") << endl
+    cout << _("  Delete snapshot:") << '\n'
+	 << _("\tsnapper delete <number>") << '\n'
+	 << '\n'
+	 << _("    Options for 'delete' command:") << '\n'
+	 << _("\t--sync, -s\t\t\tSync after deletion.") << '\n'
 	 << endl;
 }
 
 
+void
+filter_undeletables(ProxySnapshots& snapshots, vector<ProxySnapshots::iterator>& nums)
+{
+    vector<ProxySnapshots::const_iterator> undeletables;
+
+    undeletables.push_back(snapshots.begin());
+
+    ProxySnapshots::const_iterator default_snapshot = snapshots.getDefault();
+    if (default_snapshot != snapshots.end())
+	undeletables.push_back(default_snapshot);
+
+    ProxySnapshots::const_iterator active_snapshot = snapshots.getActive();
+    if (active_snapshot != snapshots.end())
+	undeletables.push_back(active_snapshot);
+
+    for (ProxySnapshots::const_iterator undeletable : undeletables)
+    {
+	vector<ProxySnapshots::iterator>::iterator keep = find_if(nums.begin(), nums.end(),
+	    [undeletable](ProxySnapshots::iterator it){ return undeletable->getNum() == it->getNum(); });
+
+	if (keep != nums.end())
+	{
+	    cerr << sformat(_("Cannot delete snapshot %d."), (*keep)->getNum()) << endl;
+	    nums.erase(keep);
+	}
+    }
+}
+
+
 void
 command_delete(ProxySnappers* snappers, ProxySnapper* snapper)
 {
@@ -792,6 +872,8 @@ command_delete(ProxySnappers* snappers, ProxySnapper* snapper)
 	}
     }
 
+    filter_undeletables(snapshots, nums);
+
     snapper->deleteSnapshots(nums, verbose);
 
     if (sync)
@@ -802,8 +884,8 @@ command_delete(ProxySnappers* snappers, ProxySnapper* snapper)
 void
 help_mount()
 {
-    cout << _("  Mount snapshot:") << endl
-	 << _("\tsnapper mount <number>") << endl
+    cout << _("  Mount snapshot:") << '\n'
+	 << _("\tsnapper mount <number>") << '\n'
 	 << endl;
 }
 
@@ -831,8 +913,8 @@ command_mount(ProxySnappers* snappers, ProxySnapper* snapper)
 void
 help_umount()
 {
-    cout << _("  Umount snapshot:") << endl
-	 << _("\tsnapper umount <number>") << endl
+    cout << _("  Umount snapshot:") << '\n'
+	 << _("\tsnapper umount <number>") << '\n'
 	 << endl;
 }
 
@@ -860,11 +942,11 @@ command_umount(ProxySnappers* snappers, ProxySnapper* snapper)
 void
 help_status()
 {
-    cout << _("  Comparing snapshots:") << endl
-	 << _("\tsnapper status <number1>..<number2>") << endl
-	 << endl
-	 << _("    Options for 'status' command:") << endl
-	 << _("\t--output, -o <file>\t\tSave status to file.") << endl
+    cout << _("  Comparing snapshots:") << '\n'
+	 << _("\tsnapper status <number1>..<number2>") << '\n'
+	 << '\n'
+	 << _("    Options for 'status' command:") << '\n'
+	 << _("\t--output, -o <file>\t\tSave status to file.") << '\n'
 	 << endl;
 }
 
@@ -926,13 +1008,13 @@ command_status(ProxySnappers* snappers, ProxySnapper* snapper)
 void
 help_diff()
 {
-    cout << _("  Comparing snapshots:") << endl
-	 << _("\tsnapper diff <number1>..<number2> [files]") << endl
-	 << endl
-	 << _("    Options for 'diff' command:") << endl
-	 << _("\t--input, -i <file>\t\tRead files to diff from file.") << endl
-	 << _("\t--diff-cmd <command>\t\tCommand used for comparing files.") << endl
-	 << _("\t--extensions, -x <options>\tExtra options passed to the diff command.") << endl
+    cout << _("  Comparing snapshots:") << '\n'
+	 << _("\tsnapper diff <number1>..<number2> [files]") << '\n'
+	 << '\n'
+	 << _("    Options for 'diff' command:") << '\n'
+	 << _("\t--input, -i <file>\t\tRead files to diff from file.") << '\n'
+	 << _("\t--diff-cmd <command>\t\tCommand used for comparing files.") << '\n'
+	 << _("\t--extensions, -x <options>\tExtra options passed to the diff command.") << '\n'
 	 << endl;
 }
 
@@ -993,11 +1075,11 @@ command_diff(ProxySnappers* snappers, ProxySnapper* snapper)
 void
 help_undo()
 {
-    cout << _("  Undo changes:") << endl
-	 << _("\tsnapper undochange <number1>..<number2> [files]") << endl
-	 << endl
-	 << _("    Options for 'undochange' command:") << endl
-	 << _("\t--input, -i <file>\t\tRead files for which to undo changes from file.") << endl
+    cout << _("  Undo changes:") << '\n'
+	 << _("\tsnapper undochange <number1>..<number2> [files]") << '\n'
+	 << '\n'
+	 << _("    Options for 'undochange' command:") << '\n'
+	 << _("\t--input, -i <file>\t\tRead files for which to undo changes from file.") << '\n'
 	 << endl;
 }
 
@@ -1144,14 +1226,14 @@ getFilesystem(const ProxyConfig& config)
 void
 help_rollback()
 {
-    cout << _("  Rollback:") << endl
-	 << _("\tsnapper rollback [number]") << endl
-	 << endl
-	 << _("    Options for 'rollback' command:") << endl
-	 << _("\t--print-number, -p\t\tPrint number of second created snapshot.") << endl
-	 << _("\t--description, -d <description>\tDescription for snapshots.") << endl
-	 << _("\t--cleanup-algorithm, -c <algo>\tCleanup algorithm for snapshots.") << endl
-	 << _("\t--userdata, -u <userdata>\tUserdata for snapshots.") << endl
+    cout << _("  Rollback:") << '\n'
+	 << _("\tsnapper rollback [number]") << '\n'
+	 << '\n'
+	 << _("    Options for 'rollback' command:") << '\n'
+	 << _("\t--print-number, -p\t\tPrint number of second created snapshot.") << '\n'
+	 << _("\t--description, -d <description>\tDescription for snapshots.") << '\n'
+	 << _("\t--cleanup-algorithm, -c <algo>\tCleanup algorithm for snapshots.") << '\n'
+	 << _("\t--userdata, -u <userdata>\tUserdata for snapshots.") << '\n'
 	 << endl;
 }
 
@@ -1298,8 +1380,8 @@ command_rollback(ProxySnappers* snappers, ProxySnapper* snapper)
 void
 help_setup_quota()
 {
-    cout << _("  Setup quota:") << endl
-	 << _("\tsnapper setup-quota") << endl
+    cout << _("  Setup quota:") << '\n'
+	 << _("\tsnapper setup-quota") << '\n'
 	 << endl;
 }
 
@@ -1321,8 +1403,8 @@ command_setup_quota(ProxySnappers* snappers, ProxySnapper* snapper)
 void
 help_cleanup()
 {
-    cout << _("  Cleanup snapshots:") << endl
-	 << _("\tsnapper cleanup <cleanup-algorithm>") << endl
+    cout << _("  Cleanup snapshots:") << '\n'
+	 << _("\tsnapper cleanup <cleanup-algorithm>") << '\n'
 	 << endl;
 }
 
@@ -1385,8 +1467,8 @@ command_debug(ProxySnappers* snappers, ProxySnapper*)
 void
 help_xa_diff()
 {
-    cout << _("  Comparing snapshots extended attributes:") << endl
-         << _("\tsnapper xadiff <number1>..<number2> [files]") << endl
+    cout << _("  Comparing snapshots extended attributes:") << '\n'
+         << _("\tsnapper xadiff <number1>..<number2> [files]") << '\n'
          << endl;
 }
 
@@ -1490,25 +1572,25 @@ help(const list<Cmd>& cmds)
 	exit(EXIT_FAILURE);
     }
 
-    cout << _("usage: snapper [--global-options] <command> [--command-options] [command-arguments]") << endl
+    cout << _("usage: snapper [--global-options] <command> [--command-options] [command-arguments]") << '\n'
 	 << endl;
 
-    cout << _("    Global options:") << endl
-	 << _("\t--quiet, -q\t\t\tSuppress normal output.") << endl
-	 << _("\t--verbose, -v\t\t\tIncrease verbosity.") << endl
-	 << _("\t--utc\t\t\t\tDisplay dates and times in UTC.") << endl
-	 << _("\t--iso\t\t\t\tDisplay dates and times in ISO format.") << endl
-	 << _("\t--table-style, -t <style>\tTable style (integer).") << endl
-	 << _("\t--config, -c <name>\t\tSet name of config to use.") << endl
-	 << _("\t--no-dbus\t\t\tOperate without DBus.") << endl
-	 << _("\t--root, -r <path>\t\tOperate on target root (works only without DBus).") << endl
-	 << _("\t--version\t\t\tPrint version and exit.") << endl
+    cout << _("    Global options:") << '\n'
+	 << _("\t--quiet, -q\t\t\tSuppress normal output.") << '\n'
+	 << _("\t--verbose, -v\t\t\tIncrease verbosity.") << '\n'
+	 << _("\t--utc\t\t\t\tDisplay dates and times in UTC.") << '\n'
+	 << _("\t--iso\t\t\t\tDisplay dates and times in ISO format.") << '\n'
+	 << _("\t--table-style, -t <style>\tTable style (integer).") << '\n'
+	 << _("\t--config, -c <name>\t\tSet name of config to use.") << '\n'
+	 << _("\t--no-dbus\t\t\tOperate without DBus.") << '\n'
+	 << _("\t--root, -r <path>\t\tOperate on target root (works only without DBus).") << '\n'
+	 << _("\t--version\t\t\tPrint version and exit.") << '\n'
 	 << endl;
 
     for (list<Cmd>::const_iterator cmd = cmds.begin(); cmd != cmds.end(); ++cmd)
 	(*cmd->help_func)();
 
-    exit (EXIT_SUCCESS);
+    exit(EXIT_SUCCESS);
 }
 
 
